﻿using System.Collections.Generic;
using Microsoft.EntityFrameworkCore;

namespace Falcon.ModelSP
{
    /// <summary>
    /// 存储过程执行器实现
    /// </summary>
    public class FalconSPRuner:IFalconSPRuner
    {
        /// <summary>
        /// 执行存储过程
        /// </summary>
        /// <typeparam name="TPrarmType">参数类型</typeparam>
        /// <param name="db">数据上下文</param>
        /// <param name="data">存储过程参数</param>
        /// <returns>受影响的行数</returns>
        public int RunSP<TPrarmType>(DbContext db,TPrarmType data) {
            return db.RunProcuder(data);
        }
        /// <summary>
        /// 执行存储过程
        /// </summary>
        /// <typeparam name="TPrarmType">参数类型</typeparam>
        /// <typeparam name="TResultType">返回值枚举的类型</typeparam>
        /// <param name="db">数据上下文</param>
        /// <param name="data">存储过程参数</param>
        /// <returns>返回数据枚举</returns>
        public IEnumerable<TResultType> RunSP<TPrarmType, TResultType>(DbContext db,TPrarmType data)
            where TResultType : class, new() {
            return RunSPQuery<TPrarmType,TResultType>(db,data);
        }

        /// <summary>
        /// 执行存储过程
        /// </summary>
        /// <typeparam name="TPrarmType">参数类型</typeparam>
        /// <typeparam name="TResultType">返回值枚举的类型</typeparam>
        /// <param name="db">数据上下文</param>
        /// <param name="data">存储过程参数</param>
        /// <returns>返回数据枚举</returns>
        public IEnumerable<TResultType> RunSPQuery<TPrarmType, TResultType>(DbContext db,TPrarmType data)
            where TResultType : class, new() {
            return db.RunProcuder<TPrarmType,TResultType>(data);
        }

        /// <summary>
        /// 根据模型定义参数执行存储过程进行查询，参数类型必须定义FalconSPReturnTypeAttribute特性
        /// </summary>
        /// <typeparam name="TPrarmType">存储过程参数类型</typeparam>
        /// <param name="db">数据上下文</param>
        /// <param name="data">存储过程参数</param>
        /// <returns>返回类型枚举FalconSPReturnTypeAttribute定义的类型枚举。</returns>
        public IEnumerable<object> RunSPQuery<TPrarmType>(DbContext db,TPrarmType data) {
            return db.RunSPQuery(data);
        }
    }
}
